@@ -4,13 +4,17 @@ Module for communication with Sentinel Hub Opensearch service.
 For more search parameters check
 `service description <http://opensearch.sentinel-hub.com/resto/api/collections/Sentinel2/describe.xml>`__.
 """
+import datetime as dt
 import logging
+from typing import Iterable, Iterator, List, Optional, Union
 from urllib.parse import urlencode
 
 from ..config import SHConfig
 from ..constants import CRS
 from ..download import DownloadClient
-from ..time_utils import parse_time, parse_time_interval, serialize_time
+from ..geometry import BBox
+from ..time_utils import RawTimeIntervalType, RawTimeType, parse_time, parse_time_interval, serialize_time
+from ..type_utils import JsonDict
 
 LOGGER = logging.getLogger(__name__)
 
@@ -19,14 +23,12 @@ class TileMissingException(Exception):
     """This exception is raised when requested tile is missing at Sentinel Hub Opensearch service"""
 
 
-def get_tile_info_id(tile_id):
+def get_tile_info_id(tile_id: str) -> JsonDict:
     """Get basic information about image tile
 
     :param tile_id: original tile identification string provided by ESA (e.g.
         'S2A_OPER_MSI_L1C_TL_SGS__20160109T230542_A002870_T10UEV_N02.01')
-    :type tile_id: str
-    :return: dictionary with info provided by Opensearch REST service or `None` if such tile does not exist on AWS.
-    :rtype: dict or None
+    :return: dictionary with info provided by Opensearch REST service
     :raises: TileMissingException if no tile with tile ID `tile_id` exists
     """
     result_list = list(search_iter(tile_id=tile_id))
@@ -40,19 +42,16 @@ def get_tile_info_id(tile_id):
     return result_list[0]
 
 
-def get_tile_info(tile, time, aws_index=None, all_tiles=False):
+def get_tile_info(
+    tile: str, time: Union[RawTimeType, RawTimeIntervalType], aws_index: Optional[int] = None, all_tiles: bool = False
+) -> Union[JsonDict, List[JsonDict]]:
     """Get basic information about image tile
 
     :param tile: tile name (e.g. ``'T10UEV'``)
-    :type tile: str
     :param time: A single date or a time interval
-    :type time: str or (str, str) or datetime.date or (datetime.date, datetime.date)
     :param aws_index: index of tile on AWS
-    :type aws_index: int or None
     :param all_tiles: If `True` it will return list of all tiles otherwise only the first one
-    :type all_tiles: bool
-    :return: dictionary with info provided by Opensearch REST service or `None` if such tile does not exist on AWS.
-    :rtype: dict or None
+    :return: dictionary (or list of dictionaries) with info provided by Opensearch REST service
     """
     start_date, end_date = parse_time_interval(time)
 
@@ -79,74 +78,63 @@ def get_tile_info(tile, time, aws_index=None, all_tiles=False):
     return candidates[0]
 
 
-def get_area_info(bbox, date_interval, maxcc=None):
+def get_area_info(bbox: BBox, date_interval: RawTimeIntervalType, maxcc: Optional[float] = None) -> List[JsonDict]:
     """Get information about all images from specified area and time range
 
     :param bbox: bounding box of requested area
-    :type bbox: geometry.BBox
     :param date_interval: a pair of time strings in ISO8601 format
-    :type date_interval: tuple(str)
     :param maxcc: filter images by maximum percentage of cloud coverage
-    :type maxcc: float in range [0, 1] or None
-    :return: list of dictionaries containing info provided by Opensearch REST service
-    :rtype: list(dict)
+    :return: iterator of dictionaries containing info provided by Opensearch REST service
     """
     result_list = search_iter(bbox=bbox, start_date=date_interval[0], end_date=date_interval[1])
     if maxcc:
         return reduce_by_maxcc(result_list, maxcc)
-    return result_list
+    return list(result_list)
 
 
-def get_area_dates(bbox, date_interval, maxcc=None):
+def get_area_dates(bbox: BBox, date_interval: RawTimeIntervalType, maxcc: Optional[int] = None) -> List[dt.date]:
     """Get list of times of existing images from specified area and time range
 
     :param bbox: bounding box of requested area
-    :type bbox: geometry.BBox
     :param date_interval: a pair of time strings in ISO8601 format
-    :type date_interval: tuple(str)
     :param maxcc: filter images by maximum percentage of cloud coverage
-    :type maxcc: float in range [0, 1] or None
     :return: list of time strings in ISO8601 format
-    :rtype: list[datetime.datetime]
     """
 
     area_info = get_area_info(bbox, date_interval, maxcc=maxcc)
-    return sorted({parse_time(tile_info["properties"]["startDate"]) for tile_info in area_info})
+    return sorted({parse_time(tile_info["properties"]["startDate"]) for tile_info in area_info})  # type: ignore
 
 
-def reduce_by_maxcc(result_list, maxcc):
+def reduce_by_maxcc(result_list: Iterable[JsonDict], maxcc: float) -> List[JsonDict]:
     """Filter list image tiles by maximum cloud coverage
 
     :param result_list: list of dictionaries containing info provided by Opensearch REST service
-    :type result_list: list(dict)
     :param maxcc: filter images by maximum percentage of cloud coverage
-    :type maxcc: float in range [0, 1]
     :return: list of dictionaries containing info provided by Opensearch REST service
-    :rtype: list(dict)
     """
     return [tile_info for tile_info in result_list if tile_info["properties"]["cloudCover"] <= 100 * float(maxcc)]
 
 
-def search_iter(tile_id=None, bbox=None, start_date=None, end_date=None, absolute_orbit=None, config=None):
+def search_iter(
+    tile_id: Optional[str] = None,
+    bbox: Optional[BBox] = None,
+    start_date: Optional[RawTimeType] = None,
+    end_date: Optional[RawTimeType] = None,
+    absolute_orbit: Optional[int] = None,
+    config: Optional[SHConfig] = None,
+) -> Iterator[JsonDict]:
     """A generator function that implements OpenSearch search queries and returns results
 
     All parameters for search are optional.
 
     :param tile_id: original tile identification string provided by ESA (e.g.
                     'S2A_OPER_MSI_L1C_TL_SGS__20160109T230542_A002870_T10UEV_N02.01')
-    :type tile_id: str
     :param bbox: bounding box of requested area
-    :type bbox: geometry.BBox
     :param start_date: beginning of time range
-    :type start_date: str
     :param end_date: end of time range
-    :type end_date: str
     :param absolute_orbit: An absolute orbit number of Sentinel-2 L1C products as defined by ESA
-    :type absolute_orbit: int
     :return: An iterator returning dictionaries with info provided by Sentinel Hub OpenSearch REST service
-    :rtype: Iterator[dict]
     :param config: A custom instance of config class to override parameters from the saved configuration.
-    :type config: SHConfig or None
     """
     config = config or SHConfig()
 
@@ -168,7 +156,7 @@ def search_iter(tile_id=None, bbox=None, start_date=None, end_date=None, absolut
         url = f"{config.opensearch_url}/search.json?{urlencode(url_params)}"
         LOGGER.debug("URL=%s", url)
 
-        response = client.get_json(url)
+        response = client.get_json_dict(url)
         for tile_info in response["features"]:
             yield tile_info
 
@@ -177,22 +165,22 @@ def search_iter(tile_id=None, bbox=None, start_date=None, end_date=None, absolut
         start_index += config.max_opensearch_records_per_query
 
 
-def _prepare_url_params(tile_id, bbox, end_date, start_date, absolute_orbit):
+def _prepare_url_params(
+    tile_id: Optional[str],
+    bbox: Optional[BBox],
+    end_date: Optional[dt.date],
+    start_date: Optional[dt.date],
+    absolute_orbit: Optional[int],
+) -> JsonDict:
     """Constructs dict with URL params
 
     :param tile_id: original tile identification string provided by ESA (e.g.
                     'S2A_OPER_MSI_L1C_TL_SGS__20160109T230542_A002870_T10UEV_N02.01')
-    :type tile_id: str
     :param bbox: bounding box of requested area in WGS84 CRS
-    :type bbox: geometry.BBox
     :param start_date: beginning of time range
-    :type start_date: datetime.date
     :param end_date: end of time range
-    :type end_date: datetime.date
     :param absolute_orbit: An absolute orbit number of Sentinel-2 L1C products as defined by ESA
-    :type absolute_orbit: int
     :return: dictionary with parameters as properties when arguments not None
-    :rtype: dict
     """
     url_params = {
         "identifier": tile_id,
