@@ -3,9 +3,11 @@ Module for creating .SAFE structure with data collected from AWS
 """
 
 import warnings
+from typing import Any, List, Optional, Tuple
 
 from ..constants import MimeType
 from ..data_collections import DataCollection
+from ..download.request import DownloadRequest
 from ..exceptions import SHRuntimeWarning
 from .client import AwsDownloadClient
 from .constants import AwsConstants, EsaSafeType
@@ -15,11 +17,10 @@ from .data import AwsProduct, AwsTile
 class SafeProduct(AwsProduct):
     """Class implementing transformation of Sentinel-2 satellite products from AWS into .SAFE structure"""
 
-    def get_requests(self):
+    def get_requests(self) -> Tuple[List[DownloadRequest], List[str]]:
         """Creates product structure and returns list of files for download
 
         :return: list of download requests
-        :rtype: list(download.DownloadRequest)
         """
         safe = self.get_safe_struct()
 
@@ -28,11 +29,10 @@ class SafeProduct(AwsProduct):
         self.sort_download_list()
         return self.download_list, self.folder_list
 
-    def get_safe_struct(self):
+    def get_safe_struct(self) -> dict:
         """Describes a structure inside tile folder of ESA product .SAFE structure
 
         :return: nested dictionaries representing .SAFE structure
-        :rtype: dict
         """
         main_folder = self.get_main_folder()
         safe = {
@@ -58,9 +58,9 @@ class SafeProduct(AwsProduct):
             )
         return safe
 
-    def _get_datastrip_substruct(self):
+    def _get_datastrip_substruct(self) -> dict:
         """Builds a datastrip subfolder structure of .SAFE format."""
-        datastrip_safe = {}
+        datastrip_safe: dict = {}
         datastrip_list = self.get_datastrip_list()
         for datastrip_folder, datastrip_url in datastrip_list:
             datastrip_safe[datastrip_folder] = {AwsConstants.QI_DATA: {}}
@@ -83,7 +83,7 @@ class SafeProduct(AwsProduct):
 
         return datastrip_safe
 
-    def _get_granule_substruct(self):
+    def _get_granule_substruct(self) -> dict:
         """Builds a granule subfolder structure of .SAFE format."""
         granule_safe = {}
         for tile_info in self.product_info["tiles"]:
@@ -104,17 +104,15 @@ class SafeProduct(AwsProduct):
 
         return granule_safe
 
-    def get_main_folder(self):
+    def get_main_folder(self) -> str:
         """
         :return: name of main folder
-        :rtype: str
         """
         return f"{self.product_id}.SAFE"
 
-    def get_datastrip_list(self):
+    def get_datastrip_list(self) -> List[Tuple[str, str]]:
         """
         :return: list of datastrips folder names and urls from `productInfo.json` file
-        :rtype: list((str, str))
         """
         datastrips = self.product_info["datastrips"]
         return [
@@ -122,23 +120,19 @@ class SafeProduct(AwsProduct):
             for datastrip in datastrips
         ]
 
-    def get_datastrip_name(self, datastrip):
+    def get_datastrip_name(self, datastrip: str) -> str:
         """
         :param datastrip: name of datastrip
-        :type datastrip: str
         :return: name of datastrip folder
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             return datastrip
         return "_".join(datastrip.split("_")[4:-1])
 
-    def get_datastrip_metadata_name(self, datastrip_folder):
+    def get_datastrip_metadata_name(self, datastrip_folder: str) -> str:
         """
         :param datastrip_folder: name of datastrip folder
-        :type datastrip_folder: str
         :return: name of datastrip metadata file
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             name = _edit_name(datastrip_folder, "MTD", delete_end=True)
@@ -146,10 +140,9 @@ class SafeProduct(AwsProduct):
             name = "MTD_DS"
         return f"{name}.{MimeType.XML.value}"
 
-    def get_product_metadata_name(self):
+    def get_product_metadata_name(self) -> str:
         """
         :return: name of product metadata file
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             name = _edit_name(self.product_id, "MTD", "SAFL1C")
@@ -157,23 +150,21 @@ class SafeProduct(AwsProduct):
             name = f'MTD_{self.product_id.split("_")[1]}'
         return f"{name}.{MimeType.XML.value}"
 
-    def get_report_name(self):
+    def get_report_name(self) -> str:
         """
         :return: name of the report file of L2A products
-        :rtype: str
         """
         return f"{self.product_id}_{self.get_report_time()}_report.{MimeType.XML.value}"
 
-    def get_report_time(self):
+    def get_report_time(self) -> str:
         """Returns time when the L2A processing started and reports was created.
         :return: String in a form YYYYMMDDTHHMMSS
-        :rtype: str
         """
         client = AwsDownloadClient(config=self.config)
         tree = client.get_xml(self.get_url(AwsConstants.REPORT))
 
         try:
-            timestamp = tree.find("check/inspection").attrib["execution"]
+            timestamp = tree.find("check/inspection").attrib["execution"]  # type: ignore
             return timestamp.split(",")[0].replace(" ", "T").replace(":", "").replace("-", "")
         except AttributeError:
             warnings.warn("Could not obtain the L2A report creation time", category=SHRuntimeWarning)
@@ -183,17 +174,16 @@ class SafeProduct(AwsProduct):
 class SafeTile(AwsTile):
     """Class implementing transformation of Sentinel-2 satellite tiles from AWS into .SAFE structure"""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any):
         """Initialization parameters are inherited from parent class"""
         super().__init__(*args, **kwargs)
 
         self.tile_id = self.get_tile_id()
 
-    def get_requests(self):
+    def get_requests(self) -> Tuple[List[DownloadRequest], List[str]]:
         """Creates tile structure and returns list of files for download.
 
         :return: list of download requests for
-        :rtype: list(download.DownloadRequest)
         """
         safe = self.get_safe_struct()
 
@@ -202,11 +192,10 @@ class SafeTile(AwsTile):
         self.sort_download_list()
         return self.download_list, self.folder_list
 
-    def get_safe_struct(self):
+    def get_safe_struct(self) -> dict:
         """Describes a structure inside tile folder of ESA product .SAFE structure.
 
         :return: nested dictionaries representing .SAFE structure
-        :rtype: dict
         """
         return {
             self.get_main_folder(): {
@@ -217,7 +206,7 @@ class SafeTile(AwsTile):
             }
         }
 
-    def _get_aux_substruct(self):
+    def _get_aux_substruct(self) -> dict:
         """Builds an auxiliary data subfolder structure of .SAFE format.
 
         Note: Old products also have DEM and MSI in aux folder which are not reconstructed here.
@@ -247,9 +236,9 @@ class SafeTile(AwsTile):
 
         return aux_safe
 
-    def _get_image_substruct(self):
+    def _get_image_substruct(self) -> dict:
         """Builds the part of structure of .SAFE format that contains satellite imagery."""
-        img_safe = {}
+        img_safe: dict = {}
 
         if self.data_collection is DataCollection.SENTINEL2_L1C:
             for band in self.bands:
@@ -270,7 +259,7 @@ class SafeTile(AwsTile):
 
         return img_safe
 
-    def _get_qi_substruct(self):
+    def _get_qi_substruct(self) -> dict:
         """Builds a quality-indicators data subfolder structure of .SAFE format."""
         qi_safe = self._get_reports_substruct()
 
@@ -311,9 +300,9 @@ class SafeTile(AwsTile):
 
         return qi_safe
 
-    def _get_reports_substruct(self):
+    def _get_reports_substruct(self) -> dict:
         """Builds a substructure of .SAFE format with reports."""
-        reports_safe = {}
+        reports_safe: dict = {}
         if not self.has_reports():
             return reports_safe
 
@@ -334,11 +323,10 @@ class SafeTile(AwsTile):
 
         return reports_safe
 
-    def get_tile_id(self):
+    def get_tile_id(self) -> str:
         """Creates ESA tile ID
 
         :return: ESA tile ID
-        :rtype: str
         """
         client = AwsDownloadClient(config=self.config)
         tree = client.get_xml(self.get_url(AwsConstants.METADATA))
@@ -348,7 +336,7 @@ class SafeTile(AwsTile):
             if (self.data_collection is DataCollection.SENTINEL2_L2A and "00.01" < self.baseline <= "02.06")
             else "TILE_ID"
         )
-        tile_id = tree[0].find(tile_id_tag).text
+        tile_id = tree[0].find(tile_id_tag).text  # type: ignore
         if self.safe_type is EsaSafeType.OLD_TYPE:
             return tile_id
 
@@ -364,40 +352,35 @@ class SafeTile(AwsTile):
 
         return "_".join([info[3], info[-2], info[-3], tile_id_time])
 
-    def get_sensing_time(self):
+    def get_sensing_time(self) -> str:
         """
         :return: Exact tile sensing time
-        :rtype: str
         """
         return self.tile_info["timestamp"].split(".")[0].replace("-", "").replace(":", "")
 
-    def get_datastrip_time(self):
+    def get_datastrip_time(self) -> str:
         """
         :return: Exact datastrip time
-        :rtype: str
         """
         # S2A_OPER_MSI_L1C_DS_EPAE_20181119T061056_S20181119T031012_N02.07 -> 20181119T031012
         # S2A_OPER_MSI_L1C_DS_EPA__20190225T132350_S20190129T144524_N02.07 -> 20190129T144524
         return self.tile_info["datastrip"]["id"].replace("__", "_").split("_")[7].lstrip("S")
 
-    def get_datatake_time(self):
+    def get_datatake_time(self) -> str:
         """
         :return: Exact time of datatake
-        :rtype: str
         """
         return self.tile_info["productName"].split("_")[2]
 
-    def get_main_folder(self):
+    def get_main_folder(self) -> str:
         """
         :return: name of tile folder
-        :rtype: str
         """
         return self.tile_id
 
-    def get_tile_metadata_name(self):
+    def get_tile_metadata_name(self) -> str:
         """
         :return: name of tile metadata file
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             name = _edit_name(self.tile_id, "MTD", delete_end=True)
@@ -405,10 +388,9 @@ class SafeTile(AwsTile):
             name = "MTD_TL"
         return f"{name}.xml"
 
-    def get_aux_data_name(self):
+    def get_aux_data_name(self) -> str:
         """
         :return: name of auxiliary data file
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             # this is not correct, but we cannot reconstruct last two timestamps in auxiliary data file name
@@ -416,14 +398,11 @@ class SafeTile(AwsTile):
             return "AUX_ECMWFT"
         return "AUX_ECMWFT"
 
-    def get_img_name(self, band, resolution=None):
+    def get_img_name(self, band: str, resolution: Optional[str] = None) -> str:
         """
         :param band: band name
-        :type band: str
         :param resolution: Specifies the resolution in case of Sentinel-2 L2A products
-        :type resolution: str or None
         :return: name of band image file
-        :rtype: str
         """
         band = band.split("/")[-1]
         if self.safe_type is EsaSafeType.OLD_TYPE:
@@ -436,16 +415,12 @@ class SafeTile(AwsTile):
             name = f"L2A_{name}"
         return f"{name}.jp2"
 
-    def get_qi_name(self, qi_type, band="B00", data_format=MimeType.GML):
+    def get_qi_name(self, qi_type: str, band: str = "B00", data_format: MimeType = MimeType.GML) -> str:
         """
         :param qi_type: type of quality indicator
-        :type qi_type: str
         :param band: band name
-        :type band: str
         :param data_format: format of the file
-        :type data_format: MimeType
         :return: name of gml file
-        :rtype: str
         """
         band = band.split("/")[-1]
         if self.safe_type is EsaSafeType.OLD_TYPE:
@@ -457,10 +432,9 @@ class SafeTile(AwsTile):
             name = f"MSK_{qi_type}_{band}"
         return f"{name}.{data_format.value}"
 
-    def get_preview_name(self):
+    def get_preview_name(self) -> str:
         """Returns .SAFE name of full resolution L1C preview
         :return: name of preview file
-        :rtype: str
         """
         if self.safe_type is EsaSafeType.OLD_TYPE:
             name = _edit_name(self.tile_id, AwsConstants.PVI, delete_end=True)
@@ -469,19 +443,14 @@ class SafeTile(AwsTile):
         return f"{name}.jp2"
 
 
-def _edit_name(name, code, add_code=None, delete_end=False):
+def _edit_name(name: str, code: str, add_code: Optional[str] = None, delete_end: bool = False) -> str:
     """Helping function for creating file names in .SAFE format
 
     :param name: initial string
-    :type name: str
     :param code:
-    :type code: str
     :param add_code:
-    :type add_code: str or None
     :param delete_end:
-    :type delete_end: bool
     :return: edited string
-    :rtype: str
     """
     info = name.split("_")
     info[2] = code
