@@ -4,7 +4,7 @@ Module defining data collections
 import warnings
 from dataclasses import dataclass, field, fields
 from enum import Enum, EnumMeta
-from typing import Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple
 
 from aenum import extend_enum
 
@@ -100,7 +100,7 @@ class OrbitDirection:
     BOTH = "BOTH"
 
 
-def _shallow_asdict(dataclass_instance):
+def _shallow_asdict(dataclass_instance: Any) -> Dict[str, Any]:
     """Returns a dictionary of fields and values, but is not recursive and does not deepcopy like `asdict`"""
     # This definition needs to be above the class definitions in the file
     return {field.name: getattr(dataclass_instance, field.name) for field in fields(dataclass_instance)}
@@ -109,7 +109,7 @@ def _shallow_asdict(dataclass_instance):
 class _DataCollectionMeta(EnumMeta):
     """Metaclass that builds DataCollection class enums"""
 
-    def __getattribute__(cls, item, *args, **kwargs):
+    def __getattribute__(cls, item: str) -> Any:
         """This is executed whenever `DataCollection.SOMETHING` is called
 
         Extended method handles cases where a collection has been renamed. It provides a new collection and raises a
@@ -125,9 +125,9 @@ class _DataCollectionMeta(EnumMeta):
             )
             warnings.warn(message, category=SHDeprecationWarning)
 
-        return super().__getattribute__(item, *args, **kwargs)
+        return super().__getattribute__(item)
 
-    def __call__(cls, value, *args, **kwargs):
+    def __call__(cls, value, *args, **kwargs):  # type: ignore
         """This is executed whenever `DataCollection('something')` is called
 
         This solves a problem of pickling a custom DataCollection and unpickling it in another process
@@ -167,26 +167,25 @@ class DataCollectionDefinition:
     # The following parameter is used to preserve custom DataCollection name during pickling and unpickling process:
     _name: Optional[str] = field(default=None, compare=False)
 
-    def __post_init__(self):
+    def __post_init__(self):  # type: ignore
         """In case a list of bands or metabands has been given this makes sure to cast it into a tuple"""
         if isinstance(self.bands, list):
             object.__setattr__(self, "bands", tuple(self.bands))
         if isinstance(self.metabands, list):
             object.__setattr__(self, "metabands", tuple(self.metabands))
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """A nicer representation of parameters that define a data collection"""
         valid_params = {name: value for name, value in _shallow_asdict(self).items() if value is not None}
         params_repr = "\n  ".join(f"{name}: {value}" for name, value in valid_params.items() if name != "_name")
         return f"{self.__class__.__name__}(\n  {params_repr}\n)"
 
-    def derive(self, **params):
+    def derive(self, **params: Any) -> "DataCollectionDefinition":
         """Create a new data collection definition from current definition and parameters that override current
         parameters
 
         :param params: Any of DataCollectionDefinition attributes
         :return: A new data collection definition
-        :rtype: DataCollectionDefinition
         """
         derived_params = _shallow_asdict(self)
         derived_params.update(params)
@@ -424,96 +423,59 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         has_cloud_coverage=True,
     )
 
-    # EOCloud collections (which are only available on a development eocloud service):
-    LANDSAT5 = DataCollectionDefinition(
-        wfs_id="L5.TILE",
-        service_url=ServiceUrl.EOCLOUD,
-        processing_level=_ProcessingLevel.GRD,
-    )
-    LANDSAT7 = DataCollectionDefinition(
-        wfs_id="L7.TILE",
-        service_url=ServiceUrl.EOCLOUD,
-        processing_level=_ProcessingLevel.GRD,
-    )
-    ENVISAT_MERIS = DataCollectionDefinition(
-        wfs_id="ENV.TILE",
-        service_url=ServiceUrl.EOCLOUD,
-        collection_type=_CollectionType.ENVISAT_MERIS,
-    )
-
     # pylint: disable=too-many-locals
     @classmethod
     def define(
         cls,
-        name,
+        name: str,
         *,
-        api_id=None,
-        catalog_id=None,
-        wfs_id=None,
-        service_url=None,
-        collection_type=None,
-        sensor_type=None,
-        processing_level=None,
-        swath_mode=None,
-        polarization=None,
-        resolution=None,
-        orbit_direction=None,
-        timeliness=None,
-        bands=None,
-        metabands=None,
-        collection_id=None,
-        is_timeless=False,
-        has_cloud_coverage=False,
-        dem_instance=None,
-    ):
+        api_id: Optional[str] = None,
+        catalog_id: Optional[str] = None,
+        wfs_id: Optional[str] = None,
+        service_url: Optional[str] = None,
+        collection_type: Optional[str] = None,
+        sensor_type: Optional[str] = None,
+        processing_level: Optional[str] = None,
+        swath_mode: Optional[str] = None,
+        polarization: Optional[str] = None,
+        resolution: Optional[str] = None,
+        orbit_direction: Optional[str] = None,
+        timeliness: Optional[str] = None,
+        bands: Optional[Tuple[Band, ...]] = None,
+        metabands: Optional[Tuple[Band, ...]] = None,
+        collection_id: Optional[str] = None,
+        is_timeless: bool = False,
+        has_cloud_coverage: bool = False,
+        dem_instance: Optional[str] = None,
+    ) -> "DataCollection":
         """Define a new data collection
 
         Note that all parameters, except `name` are optional. If a data collection definition won't be used for a
         certain use case (e.g. Process API, WFS, etc.), parameters for that use case don't have to be defined
 
         :param name: A name of a new data collection
-        :type name: str
         :param api_id: An ID to be used for Sentinel Hub Process API
-        :type api_id: str or None
         :param catalog_id: An ID to be used for Sentinel Hub Catalog API
-        :type catalog_id: str or None
         :param wfs_id: An ID to be used for Sentinel Hub WFS service
-        :type wfs_id: str or None
         :param service_url: A base URL of Sentinel Hub service deployment from where to download data. If it is not
             specified, a `sh_base_url` from a config will be used by default
-        :type service_url: str or None
         :param collection_type: A collection type
-        :type collection_type: str or None
         :param sensor_type: A type of a satellite's sensor
-        :type sensor_type: str or None
         :param processing_level: A level of processing applied on satellite data
-        :type processing_level: str or None
         :param swath_mode: A swath mode of SAR sensors
-        :type swath_mode: str or None
         :param polarization: A type of polarization
-        :type polarization: str or None
         :param resolution: A type of (Sentinel-1) resolution
-        :type resolution: str or None
         :param orbit_direction: A direction of satellite's orbit by which to filter satellite's data
-        :type orbit_direction: str or None
         :param timeliness: A timeliness of data
-        :type timeliness: str or None
         :param bands: Information about data collection bands
-        :type bands: tuple(Band) or None
         :param metabands: Information about data collection metabands
-        :type metabands: tuple(Band) or None
         :param collection_id: An ID of a BYOC or BATCH collection
-        :type collection_id: str or None
         :param is_timeless: `True` if a data collection can be filtered by time dimension and `False` otherwise
-        :type is_timeless: bool
         :param has_cloud_coverage: `True` if data collection can be filtered by cloud coverage percentage and `False`
             otherwise
-        :type has_cloud_coverage: bool
         :param dem_instance: one of the options listed in
             `DEM documentation <https://docs.sentinel-hub.com/api/latest/data/dem/#deminstance>`__
-        :type dem_instance: str or None
         :return: A new data collection
-        :rtype: DataCollection
         """
         definition = DataCollectionDefinition(
             api_id=api_id,
@@ -539,14 +501,12 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         cls._try_add_data_collection(name, definition)
         return cls(definition)
 
-    def define_from(self, name, **params):
+    def define_from(self, name: str, **params: Any) -> "DataCollection":
         """Define a new data collection from an existing one
 
         :param name: A name of a new data collection
-        :type name: str
         :param params: Any parameter to override current data collection parameters
         :return: A new data collection
-        :rtype: DataCollection
         """
         definition = self.value
         new_definition = definition.derive(**params, _name=name)
@@ -555,7 +515,7 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return DataCollection(new_definition)
 
     @classmethod
-    def _try_add_data_collection(cls, name, definition):
+    def _try_add_data_collection(cls, name: str, definition: DataCollectionDefinition) -> None:
         """Tries adding a new data collection definition. If the exact enum has already been defined then it won't do
         anything. However, if either a name or a definition has already been matched with another name or definition
         then it will raise an error.
@@ -581,14 +541,12 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         )
 
     @classmethod
-    def define_byoc(cls, collection_id, **params):
+    def define_byoc(cls, collection_id: str, **params: Any) -> "DataCollection":
         """Defines a BYOC data collection
 
         :param collection_id: An ID of a data collection
-        :type collection_id: str
         :param params: Any parameter to override default BYOC data collection parameters
         :return: A new data collection
-        :rtype: DataCollection
         """
         params["name"] = params.get("name", f"BYOC_{collection_id}")
         params["api_id"] = params.get("api_id", f"byoc-{collection_id}")
@@ -599,14 +557,12 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return cls.define(**params)
 
     @classmethod
-    def define_batch(cls, collection_id, **params):
+    def define_batch(cls, collection_id: str, **params: Any) -> "DataCollection":
         """Defines a BATCH data collection
 
         :param collection_id: An ID of a data collection
-        :type collection_id: str
         :param params: Any parameter to override default BATCH data collection parameters
         :return: A new data collection
-        :rtype: DataCollection
         """
         params["name"] = params.get("name", f"BATCH_{collection_id}")
         params["api_id"] = params.get("api_id", f"batch-{collection_id}")
@@ -617,11 +573,10 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return cls.define(**params)
 
     @property
-    def api_id(self):
+    def api_id(self) -> str:
         """Provides a Sentinel Hub Process API identifier or raises an error if it is not defined
 
         :return: An identifier
-        :rtype: str
         :raises: ValueError
         """
         if self.value.api_id is None:
@@ -629,11 +584,10 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return self.value.api_id
 
     @property
-    def catalog_id(self):
+    def catalog_id(self) -> str:
         """Provides a Sentinel Hub Catalog API identifier or raises an error if it is not defined
 
         :return: An identifier
-        :rtype: str
         :raises: ValueError
         """
         if self.value.catalog_id is not None:
@@ -644,11 +598,10 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         raise ValueError(f"Data collection {self.name} is missing a Sentinel Hub Catalog API identifier")
 
     @property
-    def wfs_id(self):
+    def wfs_id(self) -> str:
         """Provides a Sentinel Hub WFS identifier or raises an error if it is not defined
 
         :return: An identifier
-        :rtype: str
         :raises: ValueError
         """
         if self.value.wfs_id is None:
@@ -656,11 +609,10 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return self.value.wfs_id
 
     @property
-    def bands(self):
+    def bands(self) -> Tuple[Band, ...]:
         """Provides band information available for the data collection
 
         :return: A tuple of band info
-        :rtype: tuple(str)
         :raises: ValueError
         """
         if self.value.bands is None:
@@ -668,18 +620,17 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return self.value.bands
 
     @property
-    def metabands(self):
+    def metabands(self) -> Tuple[Band, ...]:
         """Provides metaband information available for the data collection
 
         :return: A tuple of metaband info
-        :rtype: tuple(str)
         :raises: ValueError
         """
         if self.value.metabands is None:
             raise ValueError(f"Data collection {self.name} does not define metabands")
         return self.value.metabands
 
-    def __getattr__(self, item, *args, **kwargs):
+    def __getattr__(self, item: str) -> Any:
         """The following insures that any attribute from DataCollectionDefinition, which is already not a
         property or an attribute of DataCollection, becomes an attribute of DataCollection
         """
@@ -688,42 +639,38 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
             if item in definition_dict:
                 return definition_dict[item]
 
-        return super().__getattribute__(item, *args, **kwargs)
+        return super().__getattribute__(item)
 
     @property
-    def is_sentinel1(self):
+    def is_sentinel1(self) -> bool:
         """Checks if data collection is a Sentinel-1 collection type
 
         Example: ``DataCollection.SENTINEL1_IW.is_sentinel1``
 
         :return: `True` if collection is Sentinel-1 collection type and `False` otherwise
-        :rtype: bool
         """
         return self.collection_type == _CollectionType.SENTINEL1
 
     @property
-    def is_byoc(self):
+    def is_byoc(self) -> bool:
         """Checks if data collection is a BYOC collection type
 
         :return: `True` if collection is a BYOC collection type and `False` otherwise
-        :rtype: bool
         """
         return self.collection_type == _CollectionType.BYOC
 
     @property
-    def is_batch(self):
+    def is_batch(self) -> bool:
         """Checks if data collection is a batch collection type
 
         :return: `True` if collection is a batch collection type and `False` otherwise
-        :rtype: bool
         """
         return self.collection_type == _CollectionType.BATCH
 
-    def contains_orbit_direction(self, orbit_direction):
+    def contains_orbit_direction(self, orbit_direction: str) -> bool:
         """Checks if a data collection contains given orbit direction
 
         :param orbit_direction: An orbit direction
-        :type orbit_direction: string
         :return: `True` if data collection contains the orbit direction
         :return: bool
         """
@@ -733,28 +680,27 @@ class DataCollection(Enum, metaclass=_DataCollectionMeta):
         return orbit_direction.upper() == defined_direction.upper()
 
     @classmethod
-    def get_available_collections(cls, config=None):
+    def get_available_collections(cls, config: Optional[SHConfig] = None) -> List["DataCollection"]:
         """Returns which data collections are available for configured Sentinel Hub OGC URL
 
         :param config: A custom instance of config class to override parameters from the saved configuration.
-        :type config: SHConfig or None
         :return: List of available data collections
-        :rtype: list(DataCollection)
         """
-        config = config or SHConfig()
-        is_eocloud = config.has_eocloud_url()
-
-        return [
-            data_collection
-            for data_collection in cls
-            if (data_collection.service_url == ServiceUrl.EOCLOUD) == is_eocloud
-        ]
+        if config is not None:
+            warnings.warn(
+                "Parameter config is deprecated, and will be removed in future.", category=SHDeprecationWarning
+            )
+        return list(cls)
 
 
 DataSource = DataCollection
 
 
-def handle_deprecated_data_source(data_collection, data_source, default=None):
+def handle_deprecated_data_source(
+    data_collection: Optional[DataCollection],
+    data_source: Optional[DataCollection],
+    default: Optional[DataCollection] = None,
+) -> DataCollection:
     """Joins parameters used to specify a data collection. In case data_source is given it raises a warning. In case
     both are given it raises an error. In case neither are given but there is a default collection it raises another
     warning.
@@ -774,4 +720,4 @@ def handle_deprecated_data_source(data_collection, data_source, default=None):
         )
         return default
 
-    return data_collection or data_source
+    return data_collection or data_source  # type: ignore
